class ServerError(Exception):
    """Класс обработки ошибок ссоединения с сервером на стороне пользователя"""
    def __init__(self, text):
        self.text = text

    def __str__(self):
        return self.text



import sys
from hashlib import pbkdf2_hmac
from binascii import hexlify

from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import QMainWindow, QAction, qApp, QLabel, QTableView, QApplication, QDialog, QPushButton, \
    QLineEdit, QFileDialog, QMessageBox
from PyQt5.QtGui import QStandardItemModel, QStandardItem


def create_connections_model(database):
    """Метод создания GUI модели таблицы подключившехся клиентов"""
    conn_list = database.get_active_list()
    model = QStandardItemModel()
    model.setHorizontalHeaderLabels(['Клиент', 'IP адрес'])
    for row in conn_list:
        client, ip = row
        client = QStandardItem(client)
        client.setEditable(False)
        ip = QStandardItem(ip)
        ip.setEditable(False)
        model.appendRow([client, ip])
    return model


def create_stat_model(database):
    """Метод создания GUI модели таблицы истории подключений клиентов"""
    history_list = database.get_history()
    model = QStandardItemModel()
    model.setHorizontalHeaderLabels(['Клиент', 'IP адрес', 'Дата входа'])
    for row in history_list:
        client, date, ip = row
        client = QStandardItem(client)
        client.setEditable(False)
        date = QStandardItem(date)
        date.setEditable(False)
        ip = QStandardItem(str(ip))
        ip.setEditable(False)
        model.appendRow([client, date, ip])
    return model


def create_clients_list(database):
    """Метод создания GUI модели таблицы всех клиентов"""
    clients = database.get_all_client()
    model = QStandardItemModel()
    model.setHorizontalHeaderLabels(['Клиент'])
    for client in clients:
        client = QStandardItem(client)
        client.setEditable(False)
        model.appendRow([client])
    return model


class MainWindow(QMainWindow):
    """Класс главного окна GUI приложения Сервера"""
    def __init__(self):
        super().__init__()
        self.initUI()

    def initUI(self):

        exit_action = QAction('Выход', self)
        exit_action.setShortcut('Ctrl+Q')
        exit_action.triggered.connect(qApp.quit)

        self.refresh_button = QAction('Обновить список', self)
        self.config_btn = QAction('Настройки сервера', self)
        self.client_btn = QAction('Список клиентов', self)
        self.show_history_button = QAction('История клиентов', self)

        self.statusBar()

        self.toolbar = self.addToolBar('MainBar')
        self.toolbar.addAction(exit_action)
        self.toolbar.addAction(self.refresh_button)
        self.toolbar.addAction(self.client_btn)
        self.toolbar.addAction(self.show_history_button)
        self.toolbar.addAction(self.config_btn)

        self.setFixedSize(800, 500)
        self.setWindowTitle('Server')

        self.label = QLabel('Список подключённых клиентов:', self)
        self.label.setFixedSize(240, 15)
        self.label.move(10, 35)

        self.active_clients_table = QTableView(self)
        self.active_clients_table.move(10, 55)
        self.active_clients_table.setFixedSize(780, 400)

        self.show()


class HistoryWindow(QDialog):
    """Класс окна отоброжения списка истории подключения клиентов"""
    def __init__(self):
        super().__init__()
        self.initUI()

    def initUI(self):
        self.setWindowTitle('Статистика клиентов')
        self.setFixedSize(600, 700)
        self.setAttribute(Qt.WA_DeleteOnClose)

        self.close_button = QPushButton('Закрыть', self)
        self.close_button.move(250, 650)
        self.close_button.clicked.connect(self.close)

        self.history_table = QTableView(self)
        self.history_table.move(10, 10)
        self.history_table.setFixedSize(580, 620)

        self.show()


class ClientsWindow(QDialog):
    """Класс отображения окна всех зарегестрированных клиентов, с возможностью их добавления и удаления."""
    def __init__(self, database):
        super().__init__()
        self.database = database
        self.message = QMessageBox()
        self.initUI()

    def initUI(self):
        self.setWindowTitle('Список зарегистрированных клиентов')
        self.setFixedSize(350, 470)
        self.setAttribute(Qt.WA_DeleteOnClose)

        self.btn_add_client = QPushButton('Добавить', self)
        self.btn_add_client.move(10, 440)
        self.btn_add_client.clicked.connect(self.add_client)

        self.btn_delete_client = QPushButton('Удалить', self)
        self.btn_delete_client.move(100, 440)
        self.btn_delete_client.clicked.connect(self.delete_client)

        self.close_button = QPushButton('Закрыть', self)
        self.close_button.move(250, 440)
        self.close_button.clicked.connect(self.close)

        self.client_table = QTableView(self)
        self.client_table.move(10, 10)
        self.client_table.setFixedSize(300, 420)

        self.show()

    def add_client(self):
        """Метод добавления нового ползователя"""
        global add_new_client
        add_new_client = AddClient(self.database)
        # add_new_client.show()
        add_new_client.exec_()
        self.update_table()
        # self.message.information(self, 'rrrrrrrrrrrr rrrrrrrrrrrr', 'rrrrrrrrrrrr')

    def delete_client(self):
        """Метод удаления ползователя"""
        client = self.client_table.currentIndex().data()

        if client:
            if self.message.question(
                    self,
                    'Удаление клиента',
                    f'Удалить контакт "{client}" ?',
                    QMessageBox.Yes,
                    QMessageBox.No
            ) == QMessageBox.Yes:
                self.database.delete_client(client)
                self.update_table()

    def update_table(self):
        """Метод обновления в окне списка зарегистрированных пользователей, после операции добавления или удаления"""
        self.client_table.setModel(create_clients_list(self.database))
        self.client_table.resizeColumnsToContents()
        self.client_table.resizeRowsToContents()


class ConfigWindow(QDialog):
    """Класс окна настроек"""
    def __init__(self):
        super().__init__()
        self.initUI()

    def initUI(self):
        self.setFixedSize(365, 260)
        self.setWindowTitle('Настройки сервера')

        self.db_path_label = QLabel('Путь до файла базы данных: ', self)
        self.db_path_label.move(10, 10)
        self.db_path_label.setFixedSize(240, 15)

        self.db_path = QLineEdit(self)
        self.db_path.setFixedSize(250, 20)
        self.db_path.move(10, 30)
        self.db_path.setReadOnly(True)

        self.db_path_select = QPushButton('Обзор...', self)
        self.db_path_select.move(275, 28)

        def open_file_dialog():
            global dialog
            dialog = QFileDialog(self)
            path = dialog.getExistingDirectory()
            # path = path.replace('/', '\\')
            self.db_path.insert(path)

        self.db_path_select.clicked.connect(open_file_dialog)

        self.db_file_label = QLabel('Имя файла базы данных: ', self)
        self.db_file_label.move(10, 68)
        self.db_file_label.setFixedSize(180, 15)

        self.db_file = QLineEdit(self)
        self.db_file.move(200, 66)
        self.db_file.setFixedSize(150, 20)

        self.port_label = QLabel('Номер порта для соединений:', self)
        self.port_label.move(10, 108)
        self.port_label.setFixedSize(180, 15)

        self.port = QLineEdit(self)
        self.port.move(200, 108)
        self.port.setFixedSize(150, 20)

        self.ip_label = QLabel('С какого IP принимаем соединения:', self)
        self.ip_label.move(10, 148)
        self.ip_label.setFixedSize(180, 15)

        self.ip_label_note = QLabel(' оставьте это поле пустым, чтобы\n принимать соединения с любых адресов.', self)
        self.ip_label_note.move(10, 168)
        self.ip_label_note.setFixedSize(500, 30)

        self.ip = QLineEdit(self)
        self.ip.move(200, 148)
        self.ip.setFixedSize(150, 20)

        self.save_btn = QPushButton('Сохранить', self)
        self.save_btn.move(190, 220)

        self.close_button = QPushButton('Закрыть', self)
        self.close_button.move(275, 220)
        self.close_button.clicked.connect(self.close)

        self.show()


class AddClient(QDialog):
    """Класс окна добавления нового пользовтеля"""
    def __init__(self, database):
        super().__init__()
        self.database = database
        self.initUI()

    def initUI(self):

        self.message = QMessageBox()

        self.setWindowTitle('Добавление нового клиента')
        self.setFixedSize(350, 200)
        self.setAttribute(Qt.WA_DeleteOnClose)

        self.close_button = QPushButton('Закрыть', self)
        self.close_button.move(250, 440)
        self.close_button.clicked.connect(self.close)

        self.label_nickname = QLabel('Nickname', self)
        self.label_nickname.setGeometry(20, 50, 100, 20)

        self.edit_nickname = QLineEdit(self)
        self.edit_nickname.setGeometry(130, 50, 160, 25)

        self.label_password_1 = QLabel('Password', self)
        self.label_password_1.setGeometry(20, 90, 100, 20)

        self.edit_password_1 = QLineEdit(self)
        self.edit_password_1.setGeometry(130, 90, 160, 25)

        self.label_password_2 = QLabel('Password (Repeat)', self)
        self.label_password_2.setGeometry(20, 130, 100, 20)

        self.edit_password_2 = QLineEdit(self)
        self.edit_password_2.setGeometry(130, 130, 160, 25)

        self.btn_save = QPushButton('Сохранить', self)
        self.btn_save.setGeometry(20, 170, 100, 20)
        self.btn_save.clicked.connect(self.save)

        self.btn_cancel = QPushButton('Отмена', self)
        self.btn_cancel.setGeometry(200, 170, 100, 20)
        self.btn_cancel.clicked.connect(self.close)

        self.show()

    def save(self):
        """Метод сохранения данных нового пользователя, с предваретельной проверкой корректности указанных данных"""
        if not self.edit_nickname.text() or not self.edit_password_1.text() or not self.edit_password_2.text():
            self.message.critical(self, 'Ошибка!!!', 'Ошибка! Форма не заполнена')
            # self.close()
        else:
            if self.database.get_client_by_name(self.edit_nickname.text()):
                self.message.critical(self, 'Ошибка!!!', 'Ошибка! Данный пользователь уже заергистрирован.')
            elif self.edit_password_1.text() != self.edit_password_2.text():
                self.message.critical(self, 'Ошибка!!!', 'Ошибка! Пароли не совпадают')
            else:
                passwd = self.edit_password_1.text().encode('utf-8')
                salt = self.edit_nickname.text().lower().encode('utf-8')
                passwd = pbkdf2_hmac('sha512', passwd, salt, 10000)

                self.database.add_client(self.edit_nickname.text(), hexlify(passwd))

                self.message.information(self, 'Добавление пользователя', 'Добавлено!')
                self.close()


if __name__ == '__main__':
    app = QApplication(sys.argv)
    main = MainWindow()
    app.exec_()



import logging
import inspect
from socket import socket
# import client_log_config
import logs.client_log_config

CLIENT_M = 'clientapp.py'
SERVER_M = 'serverapp.py'


logs_client = logging.getLogger('app.clientapp')
logs_server = logging.getLogger('app.serverapp')


def log(func):
    """Декоратор. Производит логирования совершенных действий на стороне сервера и пользователя """
    def wrapper(*args, **kwargs):
        received_from = inspect.stack()[1][1].split("/")[-1]
        parent_func = str(inspect.stack()[1][0]).split(' ')[-1][:-1]
        if received_from == CLIENT_M:
            logs_client.info(f'{received_from} - функция "{func.__name__}" вызвана из функции "{parent_func}"')
        elif received_from == SERVER_M:
            logs_server.info(f'{received_from} - функция "{func.__name__}" вызвана из функции "{parent_func}"')
        return func(*args, **kwargs)
    return wrapper


def login_required(func):
    """Декоратор. Проверяет, является ли пользователь авторизованным. """
    from common.variables import ACTION, PRESENCE, NICKNAME, ACCESS

    def wrapper(*args, **kwargs):
        args[1][ACCESS] = False
        if isinstance(args[2], socket):
            if args[1].get(ACTION) == PRESENCE:
                args[1][ACCESS] = True
            elif args[1].get(NICKNAME) and args[1].get(NICKNAME) in args[0].clients_name:
                args[1][ACCESS] = True
        return func(*args, **kwargs)
    return wrapper



import logging
import inspect
import sys
import os
import logging.handlers

sys.path.append('../')
PATH = os.path.dirname(os.path.abspath(__file__))
PATH = os.path.join(PATH, 'reports')
PATH = os.path.join(PATH, 'app.serverapp.log')

logs = logging.getLogger('app.serverapp')

formatter = logging.Formatter("%(asctime)s - %(levelname)s - %(message)s")
# file_hand = logging.FileHandler(f"{PATH}/app.clientapp.log", encoding='utf-8')
file_hand = logging.handlers.TimedRotatingFileHandler(PATH,
                                                      encoding='utf8',
                                                      interval=1,
                                                      when='midnight')
file_hand.setFormatter(formatter)

logs.addHandler(file_hand)
logs.setLevel(logging.DEBUG)

if __name__ == '__main__':
    logs.debug(f'режим отладки - {inspect.stack()[0][1].split("/")[-1]}')




import unittest
from client_dist.client.clientapp import validation, create_message
from server_dist.server.common.variables import PRESENCE, ANSWER, RESPONSE, ERROR, TEXT, ACTION


class TestClient(unittest.TestCase):

    def test_validation_success(self):
        data = validation({RESPONSE: 200, ANSWER: 'test'})
        self.assertEqual(data, '200: test')

    def test_validation_failed(self):
        data = validation({RESPONSE: 77, ERROR: 'Bad Request'})
        self.assertEqual(data, '400: Bad Request')

    def test_create_massages(self):
        text = create_message('test')
        self.assertEqual(text, {ACTION: PRESENCE, TEXT: 'test'})



from datetime import datetime

from sqlalchemy import create_engine, Column, Integer, String, ForeignKey, DateTime, Text
from sqlalchemy.orm import sessionmaker
from sqlalchemy.ext.declarative import declarative_base

from common.variables import DATABASE_SERVER


class DataBase:
    """Класс работы с базой данных на стороне Сервера на базе SQLite3 и SQLAlchemy"""
    Base = declarative_base()

    class Clients(Base):
        """Класс отображения Клиентов"""
        __tablename__ = 'Clients'
        id = Column(Integer, primary_key=True)
        nickname = Column(String, unique=True)
        password_hash = Column(String, unique=True)
        public_key = Column(Text)

        def __init__(self, nickname, password):
            self.nickname = nickname
            self.password_hash = password

        def __str__(self):
            return self.nickname

    class History(Base):
        """Класс отображения истории подлючавшихся Клиентов"""
        __tablename__ = 'History'
        id = Column(Integer, primary_key=True)
        client_id = Column(String, ForeignKey('Clients.id'))
        date_entry = Column(DateTime)
        ip = Column(String)

        def __init__(self, client_id, date_entry, ip):
            self.client_id = client_id
            self.date_entry = date_entry
            self.ip = ip

        def __str__(self):
            return f'Клиент {self.client_id}, ip {self.ip}, последний вход {self.date_entry}'

    class ActiveClients(Base):
        """Класс отображения активных(подключенных) Клиентов"""
        __tablename__ = 'ActiveClients'
        id = Column(Integer, primary_key=True)
        client_id = Column(String, ForeignKey('Clients.id'), unique=True)
        ip = Column(String)

        def __init__(self, client_id, ip):
            self.client_id = client_id
            self.ip = ip

        def __str__(self):
            return f'Клиент {self.client_id}, ip {self.ip}'

    class Contacts(Base):
        """Класс отображения контактов Клиента"""
        __tablename__ = 'Contacts'
        id = Column(Integer, primary_key=True)
        client_id = Column(String, ForeignKey('Clients.id'))
        contact_id = Column(String, ForeignKey('Clients.id'))

        def __init__(self, client_id, contact_id):
            self.client_id = client_id
            self.contact_id = contact_id

    def __init__(self):

        self.engine = create_engine(
            DATABASE_SERVER,
            echo=False,
            pool_recycle=7200,
            connect_args={'check_same_thread': False}
        )
        self.Base.metadata.create_all(self.engine)
        Session = sessionmaker(bind=self.engine)
        self.session = Session()
        self.session.query(self.ActiveClients).delete()
        self.session.commit()

    def get_client_by_name(self, nickname):
        """Метод получения клиента по имени"""
        return self.session.query(self.Clients).filter_by(nickname=nickname).first()

    def add_client(self, nickname, passwd):
        """Метод добавления нового клиента"""
        client = self.Clients(nickname, passwd)
        self.session.add(client)
        self.session.commit()

    def delete_client(self, nickname):
        """Метод удаления клиента"""
        self.session.query(self.Clients).filter_by(nickname=nickname).delete()
        self.session.commit()

    def client_entry(self, nickname, ip):
        """Метод добавления клиента в список активных(подключившехся) клиентов и добавление записи в историю входов"""
        client = self.session.query(self.Clients).filter_by(nickname=nickname).first()

        active = self.ActiveClients(client.id, ip)
        history = self.History(client.id, datetime.now(), ip)
        self.session.add(active)
        self.session.add(history)
        self.session.commit()

    def client_exit(self, nickname):
        """Метод удаления клиента из списка активных пользователей"""
        client = self.session.query(self.Clients).filter_by(nickname=nickname).first()
        self.session.query(self.ActiveClients).filter_by(client_id=client.id).delete()
        self.session.commit()

    def get_active_list(self):
        """Метод получения списка активных пользователей"""
        result = self.session.query(self.Clients.nickname, self.ActiveClients.ip).join(self.Clients)
        return result.all()

    def get_history(self):
        """Метод получения истории подключений клиентов"""
        result = self.session.query(self.Clients.nickname, self.History.ip, self.History.date_entry).join(self.Clients)
        return result.all()

    def get_contacts(self, nickname):
        """Метод получения списка контактов пользователя"""
        client = self.session.query(self.Clients).filter_by(nickname=nickname).first()
        query = self.session.query(self.Contacts, self.Clients.nickname).filter_by(client_id=client.id).join(self.Clients, self.Contacts.contact_id == self.Clients.id)
        return [item[1] for item in query.all()]

    def add_contact(self, client_name, contact):
        """Метод добавления нового контакта клиента"""
        client = self.session.query(self.Clients).filter_by(nickname=client_name).first()
        contact = self.session.query(self.Clients).filter_by(nickname=contact).first()

        if not contact:
            return False

        if self.session.query(self.Contacts).filter_by(client_id=client.id, contact_id=contact.id).count():
            return True

        new_contact = self.Contacts(client.id, contact.id)
        self.session.add(new_contact)
        self.session.commit()
        return True

    def delete_contact(self, client_name, contact):
        """Метод уаления контакта из списка контактов клиента"""
        client = self.session.query(self.Clients).filter_by(nickname=client_name).first()
        contact = self.session.query(self.Clients).filter_by(nickname=contact).first()

        if not contact:
            return False

        del_contact = self.session.query(self.Contacts).filter(self.Contacts.client_id == client.id,
                                                               self.Contacts.contact_id == contact.id)
        del_contact.delete()
        self.session.commit()
        return True

    def get_all_client(self):
        """Метод получения списка всех известных клиентов"""
        return [client[0] for client in self.session.query(self.Clients.nickname).all()]

    def get_all_contacts(self):
        """Метод получения списка всех контаков"""
        return self.session.query(self.Contacts).join(self.Clients)

    def check_client(self, nickname):
        """Метод проверки наличия клиента с списке зарегистрированных"""
        if self.session.query(self.Clients).filter_by(nickname=nickname).count():
            return True
        return False


if __name__ == '__main__':
    pass



ENCODING = 'utf-8'
PRESENCE = 'presence'
EXIT = 'exit'
TIME = 'time'
ACTION = 'action'
ANSWER = 'answer'
TEXT = 'text'
RESPONSE = 'response'
MESSAGE = 'message'
GET_CONTACT = 'get_contacts'
CONTACTS = 'alert'
ADD_CONTACT = 'add_contacts'
DEL_CONTACT = 'del_contacts'
CONTACT_NAME = 'contact_name'
FROM = 'from'
TO = 'to'
NICKNAME = 'nickname'
ERROR = 'error'
DEFAULT_PORT = 7777
DEFAULT_IP = '127.0.0.1'
MAX_DATA_LENGTH = 1024
DATABASE_SERVER = 'sqlite:///database_server.db3'
DATABASE_CLIENT = 'sqlite:///database_client.db3'
PUBLIC_KEY = 'public_key'
DATA = 'bin'
ACCESS = 'access'



class Port:
    """Класс проверки корректности введенного номера IP порта"""
    def __set__(self, instance, value):
        if value < 1025:
            print('Ошибка!!! Порт не может быть меньше 1025')
            exit(1)
        if value > 65536:
            print('Ошибка!!! Порт не может быть больше 65536')
            exit(1)
        instance.__dict__[self.my_attr] = value

    def __set_name__(self, owner, my_attr):
        self.my_attr = my_attr



import os.path
import sys
import logging
import inspect
import argparse
import select
import hmac
import configparser
from socket import socket, AF_INET, SOCK_STREAM, SOL_SOCKET, SO_REUSEADDR
from threading import Thread, Lock
from binascii import hexlify, a2b_base64
from os import urandom

from PyQt5.QtWidgets import QApplication, QMessageBox
from PyQt5.QtCore import QTimer

from common.variables import (DEFAULT_PORT, DEFAULT_IP, PRESENCE, RESPONSE,
                              ERROR, ACTION, MESSAGE, NICKNAME, TEXT, TO,
                              EXIT, GET_CONTACT, ADD_CONTACT, DEL_CONTACT,
                              CONTACTS, CONTACT_NAME, DATA, ACCESS)
from common.utils import receive_message, send_message
from logs.decor import log, login_required
# import logs.server_log_config
from serverapp.server_gui import (MainWindow, HistoryWindow, ConfigWindow,
                                  create_stat_model, create_connections_model,
                                  ClientsWindow, create_clients_list)
from common.metaclasses import ServerVerifier
from common.descriptrs import Port
from serverapp.database_server import DataBase

logs_server = logging.getLogger('app.serverapp')
MOD = inspect.stack()[0][1].split("/")[-1]

new_connection = False
conflag_lock = Lock()


@log
def arg_data():
    """Функция парсинга агуиментов переданных при запуске сервера из консоли"""
    parse = argparse.ArgumentParser()
    parse.add_argument('-a', default=DEFAULT_IP, help='IP adress', nargs='?')
    parse.add_argument('-p', default=DEFAULT_PORT, help='PORT', type=int, nargs='?')
    namespace = parse.parse_args(sys.argv[1:])
    ip = namespace.a
    port = namespace.p

    return ip, port


class Server(Thread, metaclass=ServerVerifier):
    """Главный класс сервера"""
    port = Port()

    def __init__(self, ip, port, database):
        self.ip = ip
        self.port = port
        self.database = database
        self.clients = []
        self.messages = []
        self.clients_name = dict()  # Список сокетов с именами клиентов. {client_name: client_socket}
        self.connection = socket(AF_INET, SOCK_STREAM)
        self.connection.setsockopt(SOL_SOCKET, SO_REUSEADDR, 1)
        self.connection.bind((self.ip, self.port))
        print(f'Запущен сервер с праметрами: ip = "{self.ip}", port = {self.port}')
        self.connection.settimeout(0.5)
        self.connection.listen(5)
        super().__init__()

    def run(self):
        """ Основной метод работы сервера"""
        global new_connection
        while True:
            try:
                client, client_address = self.connection.accept()
            except OSError:
                pass
            else:
                data = receive_message(client)
                data = self.validation(data, client)
            receive_data_lst = []
            send_data_lst = []
            errors_lst = []
            try:
                if self.clients:
                    receive_data_lst, send_data_lst, errors_lst = select.select(self.clients, self.clients, [], 0)
            except OSError:
                pass
            # Получение сообщений
            if receive_data_lst:
                for client_m in receive_data_lst:
                    try:
                        data = receive_message(client_m)
                        data = self.validation(data, client_m)
                        if ACTION in data:
                            if data[ACTION] == MESSAGE:
                                self.messages.append((data[NICKNAME], data[TEXT], data[TO]))
                                print(f'Получено сообщение от {data[NICKNAME]} для {data[TO]}')
                                logs_server.info(f'Получено сообщение от клиента {data[NICKNAME]}')
                            elif data[ACTION] == EXIT:
                                print(f'Пользователь {data[NICKNAME]} отключился')
                                logs_server.info(f'Пользователь {data[NICKNAME]} отключился')
                                self.database.client_exit(data[NICKNAME])
                                self.clients.remove(client_m)
                                del self.clients_name[data[NICKNAME]]
                                with conflag_lock:
                                    new_connection = True
                    except Exception as e:
                        logs_server.info(f'{MOD} - Ошибка получения сообщения от {client_m}; Ошибка:{e}')
                        logs_server.info(f'{MOD} - клиент {client_m} отключился')
                        for client in self.clients_name:
                            if self.clients_name[client] == client_m:
                                self.database.client_exit(client)
                                print(f'Клиент {client} отключился')
                                del self.clients_name[client]
                                break
                        self.clients.remove(client_m)
                        with conflag_lock:
                            new_connection = True
            # Отправка сообщений
            for message in self.messages:
                if message[2] not in self.clients_name.keys():  # Проверяем, есть ли пользователь с таким Именем
                    continue
                message_to_send = self.create_message(message[0], message[1])
                try:
                    send_message(self.clients_name[message[2]], message_to_send)
                except Exception as e:
                    logs_server.info(f'{MOD} - Ошибка отправки сообщения для {self.clients_name[message[2]]}; Ошибка:{e}')
                    logs_server.info(f'{MOD} - клиент {self.clients_name[message[2]]} отключился')
                    self.database.client_exit(data[NICKNAME])
                    self.clients_name[message[2]].close()
                    del self.clients_name[message[2]]
            self.messages.clear()

    @log
    @login_required
    def validation(self, data, client):
        """Метод валидации сообщений полученных от клиентов"""
        if not data[ACCESS]:
            try:
                send_message(client, {RESPONSE: 400, ERROR: 'Bad Request'})
            except OSError:
                client.close()

        global new_connection

        # Подключение клиента
        if ACTION in data and data[ACTION] == PRESENCE:
            if data[NICKNAME] in self.clients_name.keys():
                send_message(client, {RESPONSE: 400, ERROR: 'Пользователь уже подключен'})
                return

            self.authorization(data, client)

        # Обработка сообщения клиента
        elif ACTION in data and data[ACTION] == MESSAGE:
            logs_server.info(f'Получено сообщение от "{data[NICKNAME]}", для {data[TO]}')
            return {ACTION: MESSAGE, NICKNAME: data[NICKNAME], TEXT: data[TEXT], TO: data[TO]}

        #  Клиент выходит
        elif ACTION in data and data[ACTION] == EXIT:
            return {ACTION: EXIT, NICKNAME: data[NICKNAME]}

        elif ACTION in data and data[ACTION] == GET_CONTACT:
            send_message(client, {RESPONSE: 202, CONTACTS: self.database.get_contacts(data[NICKNAME])})
            return data

        elif ACTION in data and data[ACTION] == ADD_CONTACT:
            if self.database.add_contact(data[NICKNAME], data[CONTACT_NAME]):
                send_message(client, {RESPONSE: 200})
            else:
                send_message(client, {RESPONSE: 406})
            return data
        elif ACTION in data and data[ACTION] == DEL_CONTACT:
            if self.database.delete_contact(data[NICKNAME], data[CONTACT_NAME]):
                send_message(client, {RESPONSE: 200})
            else:
                send_message(client, {RESPONSE: 406})
            return data
        else:
            logs_server.warning(f'{MOD} - клиенту отправлен код 400 в функции - "{inspect.stack()[0][3]}"')
            send_message(client, {RESPONSE: 400, ERROR: 'Bad Request'})

    def authorization(self, data, client):
        """Метод авторизации клиентов"""
        global new_connection
        # Проверяем есть ли ткой пользователь
        if not self.database.check_client(data[NICKNAME]):
            try:
                send_message(client, {RESPONSE: 400, ERROR: 'Пользователь не зарегистриован'})
                print(f'Пользователь {data[NICKNAME]}:  не зарегистрирован')
            except OSError:
                pass
        else:
            # hash представление
            random_str = hexlify(urandom(64))
            message_out = {
                RESPONSE: 511,
                DATA: random_str.decode('ascii')
            }
            try:
                send_message(client, message_out)
                answer = receive_message(client)
            except OSError:
                client.close()
                print(f'Пользователь {data[NICKNAME]}:  Ошибка отправки хэш авторизации')
                return
            client_digest = a2b_base64(answer[DATA])
            if RESPONSE in answer and answer[RESPONSE] == 511:
                hash = hmac.new(
                    self.database.get_client_by_name(answer[NICKNAME]).password_hash,
                    random_str,
                    'MD5'
                )
                digest = hash.digest()
                if hmac.compare_digest(digest, client_digest):
                    self.clients_name[data[NICKNAME]] = client
                    print(f'Подключился клиент {data[NICKNAME]}')
                    with conflag_lock:
                        new_connection = True
                    ip, port = client.getpeername()
                    self.database.client_entry(data[NICKNAME], ip)
                    send_message(client, {RESPONSE: 200})
                    logs_server.info(f'Установлено соединения с клиентом "{data[NICKNAME]}", с адресом {ip}')
                    self.clients.append(client)
                else:
                    try:
                        send_message(client, {RESPONSE: 400, ERROR: 'Invalid username or password.'})
                        print(f'Пользователь {data[NICKNAME]}:  Не верный пароль')
                    except OSError:
                        client.close()
            else:
                try:
                    send_message(client, {RESPONSE: 400, ERROR: 'Invalid username or password.'})
                except OSError:
                    client.close()

    @staticmethod
    @log
    def create_message(client_from, text):
        """Метод создания текстового сообщения от клиента для клиента"""
        return {
            ACTION: MESSAGE,
            NICKNAME: client_from,
            TEXT: text
        }


if __name__ == '__main__':
    """Запуск сервера"""
    config = configparser.ConfigParser()
    path = os.path.dirname(os.path.realpath(__file__))
    config.read(f"{path}/{'serverapp.ini'}")
    database = DataBase()

    ip, port = arg_data()
    server = Server(ip, port, database)
    server.daemon = True
    server.start()

    server_app = QApplication(sys.argv)
    main_window = MainWindow()
    main_window.statusBar().showMessage('Working')

    def connection_update():
        global new_connection
        if new_connection:
            main_window.active_clients_table.setModel(create_connections_model(database))
            main_window.active_clients_table.resizeColumnsToContents()
            main_window.active_clients_table.resizeRowsToContents()
            with conflag_lock:
                new_connection = False

    def show_clients():
        global clients_list
        clients_list = ClientsWindow(database)
        clients_list.client_table.setModel(create_clients_list(database))
        clients_list.client_table.resizeColumnsToContents()
        clients_list.client_table.resizeRowsToContents()

    def show_statistics():
        global stat_window
        stat_window = HistoryWindow()
        stat_window.history_table.setModel(create_stat_model(database))
        stat_window.history_table.resizeColumnsToContents()
        stat_window.history_table.resizeRowsToContents()

    def server_config():
        global config_window
        config_window = ConfigWindow()
        config_window.db_path.insert(config['SETTINGS']['Database_path'])
        config_window.db_file.insert(config['SETTINGS']['Database_file'])
        config_window.port.insert(config['SETTINGS']['Default_port'])
        config_window.ip.insert(config['SETTINGS']['Listen_Address'])
        config_window.save_btn.clicked.connect(save_server_config)

    def save_server_config():
        global config_window
        message = QMessageBox()
        config['SETTINGS']['Database_path'] = config_window.db_path.text()
        config['SETTINGS']['Database_file'] = config_window.db_file.text()
        try:
            port = int(config_window.port.text())
        except ValueError:
            message.warning(config_window, 'Ошибка', 'Порт должен быть числом')
        else:
            config['SETTINGS']['Listen_Address'] = config_window.ip.text()
            if 1023 < port < 65536:
                config['SETTINGS']['Default_port'] = str(port)
                with open('server.ini', 'w') as conf:
                    config.write(conf)
                    message.information(config_window, 'OK', 'Настройки успешно сохранены!')
            else:
                message.warning(config_window, 'Ошибка', 'Порт должен быть от 1024 до 65536')

    timer = QTimer()
    timer.timeout.connect(connection_update)
    timer.start(1000)

    main_window.refresh_button.triggered.connect(connection_update)
    main_window.client_btn.triggered.connect(show_clients)
    main_window.show_history_button.triggered.connect(show_statistics)
    main_window.config_btn.triggered.connect(server_config)

    server_app.exec_()




import json
import sys
sys.path.append('../')
from common.variables import ENCODING, MAX_DATA_LENGTH


def receive_message(client):
    """Функция получения сообщения в формате byte и его преобразования в формат JSON"""
    data = client.recv(MAX_DATA_LENGTH)
    if isinstance(data, bytes):
        data = data.decode(ENCODING)
        data = json.loads(data)
        return data
    raise ValueError


def send_message(sock, data):
    """Функция преобразования сообщения из формата JSON в формат byte и его отправки в сокет"""
    message = json.dumps(data)
    code_message = message.encode(ENCODING)
    sock.send(code_message)



import unittest
from server_dist.server.serverapp import validation
from server_dist.server.common.variables import ACTION, PRESENCE, RESPONSE, ERROR, ANSWER


class TestServer(unittest.TestCase):
    def test_validation_success(self):
        data = validation({ACTION: PRESENCE})
        self.assertEqual(data, {RESPONSE: 200, ANSWER: 'Hello Client'})

    def test_validation_failed(self):
        data = validation('sfds')
        self.assertEqual(data, {RESPONSE: 400, ERROR: 'Bad Request'})



import dis


class ClientVerifier(type):
    """Метакласс клиента, проверяющий отсутсвие недопучтимых методов (listen, accept, socket)"""
    def __init__(cls, class_name, class_parents, class_attrs):
        for attr in class_attrs:
            try:
                ret = dis.get_instructions(class_attrs[attr])
            except TypeError:
                pass
            else:
                for i in ret:
                    if i.argval == 'listen':
                        raise TypeError('Исползован недопустимый метод "listen"')
                    elif i.argval == 'accept':
                        raise TypeError('Исползован недопустимый метод "accept"')
                    elif i.argval == 'socket':
                        raise TypeError('Исползован недопустимый метод "socket"')
        super(ClientVerifier, cls).__init__(class_name, class_parents, class_attrs)


class ServerVerifier(type):
    """Метакласс сервера, проверяющий отсутсвие недопумтимого метода connect"""
    def __init__(cls, class_name, class_parents, class_attrs):
        for attr in class_attrs:
            try:
                if attr == '__doc__':
                    continue
                ret = dis.get_instructions(class_attrs[attr])
            except TypeError:
                pass
            else:
                for i in ret:
                    if i.argval == 'connect':
                        raise TypeError('Исползован недопустимый метод "connect"')
        super(ServerVerifier, cls).__init__(class_name, class_parents, class_attrs)
